<?php

/**
 * A Magento 2 module named Lootly/OrderNotifier
 * Copyright (C) 2016  2015
 * This file included in Lootly/OrderNotifier is licensed under OSL 3.0
 *
 * http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 * Please see LICENSE.txt for the full text of the OSL 3.0 license
 */

namespace TrustSpot\Lootly\Controller\Adminhtml\System\Config;

class Verify extends \Magento\Backend\App\Action
{

    protected $resultPageFactory;

    /**
     * @var \TrustSpot\Lootly\Helper\Api
     */
    protected $orderNotifierApiHelper;

    /**
     * @var \Magento\Framework\App\Request\Http
     */
    protected $request;

    /**
     * @var \Magento\Config\Model\ResourceModel\Config
     */
    protected $_resourceConfig;

    /**
     * Constructor
     *
     * @param \Magento\Backend\App\Action\Context $context
     * @param \Magento\Framework\View\Result\PageFactory $resultPageFactory
     */
    public function __construct(
        \Magento\Backend\App\Action\Context $context,
        \TrustSpot\Lootly\Helper\Api $orderNotifierApiHelper,
        \Magento\Config\Model\ResourceModel\Config $resourceConfig,
        \Magento\Framework\App\Request\Http $request
    )
    {
        $this->orderNotifierApiHelper = $orderNotifierApiHelper;
        $this->_resourceConfig = $resourceConfig;
        $this->request = $request;
        parent::__construct($context);
    }

    /**
     * Execute view action
     *
     * @return \Magento\Framework\Controller\ResultInterface
     */
    public function execute()
    {

        $_helper = $this->orderNotifierApiHelper;
        $email = $this->request->getParam('email');
        $key = $this->request->getParam('apikey');
        $secret = $this->request->getParam('secret');
        $storeId = $this->request->getParam('storeid');
        $data = array(
            'email' => $email,
            'key' => $key,
            'secret' => $secret
        );
        if ($_helper->verify($data)) {
            /**
             * Settings are verified
             * Now saving Email and Api Key
             */
            $scope = 'default';

            if ($storeId) {
                $scope = 'store';
            } else {
                $scope = 'default';
                $storeId = 0;
            }
            $this->_resourceConfig->saveConfig(
                'lootlyordernotifier/general/email',
                $email,
                $scope,
                $storeId
            );
            $this->_resourceConfig->saveConfig(
                'lootlyordernotifier/general/api_key',
                $key,
                $scope,
                $storeId
            );
            $this->_resourceConfig->saveConfig(
                'lootlyordernotifier/general/api_secret',
                $secret,
                $scope,
                $storeId
            );
            $_helper->installApp($key, $secret);

            $this->messageManager->addSuccess('The extension is connected to your Lootly account.');
        } else {
            $this->messageManager->addError('Error! Please check your account email and api key then retry.');
        }

        $this->_redirect($this->_redirect->getRefererUrl());
    }
}
