<?php

/**
 * TrustSpot_Lootly Verfication Controller
 *
 * @category    Lootly Extensions
 * @package     TrustSpot_Lootly
 * @author      Lootly Inc
 */

namespace TrustSpot\Lootly\Controller\Create;

use Magento\Framework\App\Action\Context;

class Coupon extends \Magento\Framework\App\Action\Action implements \Magento\Framework\App\ActionInterface
{
    protected $productFactory;
    protected $api;
    protected $ruleFactory;
    protected $storeManager;
    protected $groupRepository;
    protected $searchCriteriaBuilder;
    protected $resultJsonFactory;
    protected $categoryFactory;

    public function __construct(
        Context $context,
        \Magento\Catalog\Model\ProductFactory $productFactory,
        \TrustSpot\Lootly\Helper\Api $api,
        \Magento\SalesRule\Model\RuleFactory $ruleFactory,
        \Magento\Store\Model\StoreManagerInterface $storeManagerInterface,
        \Magento\Customer\Api\GroupRepositoryInterface $groupRepository,
        \Magento\Framework\Api\SearchCriteriaBuilder $searchCriteriaBuilder,
        \Magento\Framework\Controller\Result\JsonFactory $resultJsonFactory,
        \Magento\Catalog\Model\CategoryFactory $categoryFactory
    )
    {
        $this->ruleFactory = $ruleFactory;
        $this->productFactory = $productFactory;
        $this->api = $api;
        $this->storeManager = $storeManagerInterface;
        $this->groupRepository = $groupRepository;
        $this->searchCriteriaBuilder = $searchCriteriaBuilder;
        $this->resultJsonFactory = $resultJsonFactory;
        $this->categoryFactory = $categoryFactory;
        parent::__construct($context);
    }


    /**
     * Default action
     */
    public function execute()
    {
        $postdata = $this->getRequest()->getParams();
        $q = '';
        if (isset($postdata['title'])) {
            $q = $postdata['title'];
        } elseif (isset($postdata['q'])) {
            $q = $postdata['q'];
        }
        $data = [];
        if (isset($postdata['products']) && $q) {
            $data = $this->_productRequest($q);
        } elseif (isset($postdata['categories'])) {
            $data = $this->_categoriesRequest($q);
        } elseif (isset($postdata['price-rules'])) {
            $data = $this->_createCoupon();
        }
        $result = $this->resultJsonFactory->create();
        $result->setData($data);
        return $result;
    }

    /**
     * _isAllowed
     *
     * @access protected
     * @return true
     */
    protected function _isAllowed()
    {
        return true;
    }

    protected function _productRequest($q)
    {
        $product = $this->productFactory->create();
        $collection = $product->getCollection();
        $collection->addAttributeToFilter('name', array('like' => '%' . $q . '%'));
        $collection->addAttributeToFilter('status', array('eq' => '1'));
        $collection->addAttributeToSelect('price');
        $products = [];
        foreach ($collection as $product) {
            $products[] = array(
                'id' => $product->getEntityId(),
                'title' => $product->getName(),
                /*"variants" => array(
                    "id"=> $product->getEntityId(),
                    "product_id" => $product->getEntityId(),
                    'title' => $product->getName(),
                    'price' => $product->getPrice(),
                    'sku' => $product->getSku(),
                )*/
            );
        }
        return array('products' => $products);
    }

    protected function _categoriesRequest($q)
    {
        $category = $this->categoryFactory->create();
        $collection = $category->getCollection();
        $collection->addAttributeToFilter('name', array('like' => '%' . $q . '%'));
        $categories = [];
        foreach ($collection as $category) {
            $categories[] = array(
                'id' => $category->getEntityId(),
                'title' => $category->getName(),
            );
        }
        return array('categories' => $categories);
    }

    protected function _createCoupon()
    {
        $result = [];
        $data = file_get_contents('php://input');
        $data = json_decode($data, true);
        $params = $data;
        unset($params['hmac']);
        ksort($params);
        $data1 = json_encode($params);
        $stores = $this->storeManager->getStores();
        foreach ($stores as $store) {
            $apiKey = $this->api->getApiKey($store->getId());
            if ($apiKey == $data['key']) {
                $apiSecret = $this->api->getApiSecret($store->getId());
                break;
            }
        }
        $generatedHmac = base64_encode(hash_hmac('sha256', $data1, $apiSecret, true));
        if ($generatedHmac == $data['hmac']) {
            /** @var \Magento\SalesRule\Model\Rule $model */
            $model = $this->ruleFactory->create();
            $priceRules = $this->getRequest()->getParam('price-rules');
            if ($priceRules) {
                $model->load($priceRules);
                $result = array(
                    'discount_code' => array(
                        'id' => $model->getRuleId(),
                        'price_rule_id' => $model->getRuleId(),
                        'code' => $model->getCouponCode(),
                        'usage_count' => $model->getTimesUsed()
                    )
                );
            } else {
                $dataRecived = $data['price_rule'];
                $websites = $this->storeManager->getWebsites();
                $websiteIds = [];
                foreach ($websites as $website) {
                    $websiteIds[] = $website->getWebsiteId();
                }
                $customerGroups = $this->groupRepository->getList($this->searchCriteriaBuilder->create())->getItems();
                $groups = [];
                foreach ($customerGroups as $group) {
                    $groups[] = $group->getId();
                }
                //$groups[] = 0;
                $couponType = 2;
                //$couponCode = strtoupper(substr(md5('lootly' . microtime()), 0, 10));
                $couponCode = $dataRecived['title'];
                /** @var \Magento\Framework\Stdlib\DateTime $fromDate */
                $fromDate = $this->_objectManager->create('\Magento\Framework\Stdlib\DateTime');
                $from_date = $fromDate->gmDate('Y-m-d H:i:s', strtotime($dataRecived['starts_at']));
                $to_date = new \DateTime('now+10years');
                $to_date = $to_date->format('Y-m-d H:i:s');
                $simpleFreeShipping = 0;
                if (isset($dataRecived['ends_at'])) {
                    $toDate = $this->_objectManager->create('\Magento\Framework\Stdlib\DateTime');
                    $to_date = $toDate->gmDate('Y-m-d H:i:s', strtotime($dataRecived['ends_at']));
                }

                $simpleAction = 'by_fixed';
                if ($dataRecived['value_type'] == 'percentage') {
                    $simpleAction = 'by_percent';
                } elseif ($dataRecived['value_type'] == 'fixed_amount') {
                    $simpleAction = 'cart_fixed';
                }
                $discountAmount = abs($dataRecived['value']);
                $cartCondition = array(
                    '1' => array(
                        'type' => 'Magento\SalesRule\Model\Rule\Condition\Combine',
                        'aggregator' => 'all',
                        'value' => 1,
                        'new_child' => ''
                    )
                );
                $actions = array(
                    '1' => array(
                        'type' => 'Magento\SalesRule\Model\Rule\Condition\Product\Combine',
                        'aggregator' => 'all',
                        'value' => 1,
                        'new_child' => '',
                    )
                );
                $applyToShipping = 0;
                $additionalCartConditions = [];
                if (isset($dataRecived['prerequisite_subtotal_range']) && isset($dataRecived['prerequisite_subtotal_range']['greater_than_or_equal_to'])) {
                    $additionalCartConditions[] = array(
                        'type' => 'Magento\SalesRule\Model\Rule\Condition\Address',
                        'attribute' => 'base_subtotal',
                        'operator' => '>=',
                        'value' => $dataRecived['prerequisite_subtotal_range']['greater_than_or_equal_to']
                    );
                }
                $prerequisite_customer_ids = [];
                if (isset($dataRecived['prerequisite_customer_ids']) && count($dataRecived['prerequisite_customer_ids'])) {
                    $additionalCartConditions[] = array(
                        'type' => 'Magento\SalesRule\Model\Rule\Condition\Address',
                        'attribute' => 'customer_id',
                        'operator' => '()',
                        'value' => implode(',', $dataRecived['prerequisite_customer_ids'])
                    );
                }
                if (isset($dataRecived['prerequisite_customer_emails']) && count($dataRecived['prerequisite_customer_emails'])) {
                    $emails = $dataRecived['prerequisite_customer_emails'];
                    $additionalCartConditions[] = array(
                        'type' => 'Magento\SalesRule\Model\Rule\Condition\Address',
                        'attribute' => 'email',
                        'operator' => '()',
                        'value' => implode(',', $emails)
                    );
                }
                if (isset($dataRecived['target_type']) && $dataRecived['target_type'] == 'shipping_line') {
                    if ($dataRecived['value_type'] == 'percentage' && $dataRecived['value'] == '-100.0') {
                        $applyToShipping = 1;
                        $simpleAction = 'by_fixed';
                        $discountAmount = 0;
                        /*if (isset($dataRecived['prerequisite_shipping_price_range'])) {
                            if (isset($dataRecived['prerequisite_shipping_price_range']['less_than_or_equal_to'])) {
                                $discountAmount = $dataRecived['prerequisite_shipping_price_range']['less_than_or_equal_to'];
                                $simpleAction = 'by_fixed';
                            }
                        }*/
                        /*$actions['1--1'] = array(
                            'type' => 'Magento\SalesRule\Model\Rule\Condition\Product',
                            'attribute' => 'entity_id',
                            'operator' => '==',
                            'value' => 0
                        );*/
                        $simpleFreeShipping = 1;
                    } else {
                        return array('error' => 'shipping_line items can only have percentage -100%');
                    }

                }
                $i = 1;
                foreach ($additionalCartConditions as $additionalCartCondition) {
                    $cartCondition['1--' . $i++] = $additionalCartCondition;
                }
                $discountQty = 0;
                /*if ($simpleAction == 'by_percent') {
                    $discountQty = 0;
                } else {
                    $discountQty = 1;
                }*/
                $addition = 1;
                if (isset($dataRecived['entitled_product_ids'])) {
                    //free product
                    $actions['1--'.$addition++] = array(
                        'type' => 'Magento\SalesRule\Model\Rule\Condition\Product',
                        'attribute' => 'entity_id',
                        'operator' => '()',
                        'value' => implode(',', $dataRecived['entitled_product_ids'])
                    );
                    $discountQty = 1;
                }
                if (isset($dataRecived['entitled_category_ids'])) {
                    //free product
                    $actions['1--'.$addition++] = array(
                        'type' => 'Magento\SalesRule\Model\Rule\Condition\Product',
                        'attribute' => 'category_ids',
                        'operator' => '()',
                        'value' => implode(',', $dataRecived['entitled_category_ids'])
                    );
                    /*if (!$discountAmount) {
                        $discountAmount = '100';
                    }*/
                    $discountQty = 1;
                }
                if (isset($dataRecived['excluded_product_ids'])) {
                    //free product
                    $actions['1--'.$addition++] = array(
                        'type' => 'Magento\SalesRule\Model\Rule\Condition\Product',
                        'attribute' => 'entity_id',
                        'operator' => '!()',
                        'value' => implode(',', $dataRecived['excluded_product_ids'])
                    );
                }
                if (isset($dataRecived['excluded_category_ids'])) {
                    //free product
                    $actions['1--'.$addition++] = array(
                        'type' => 'Magento\SalesRule\Model\Rule\Condition\Product',
                        'attribute' => 'category_ids',
                        'operator' => '!()',
                        'value' => implode(',', $dataRecived['excluded_category_ids'])
                    );
                }

                $data = array(
                    'name' => 'Lootly ' . $dataRecived['title'],
                    'description' => 'Generated by Lootly',
                    'is_active' => 1,
                    'website_ids' => $websiteIds,
                    'customer_group_ids' => $groups,
                    'coupon_type' => $couponType,
                    'coupon_code' => $couponCode,
                    'uses_per_coupon' => $dataRecived['usage_limit'],
                    'uses_per_customer' => $dataRecived['usage_limit'],
                    'from_date' => $from_date,
                    'to_date' => $to_date,
                    'sort_order' => 0,
                    'is_rss' => 0,
                    'rule' => array(
                        'conditions' => $cartCondition,
                        'actions' => $actions
                    ),
                    'simple_action' => $simpleAction,
                    'discount_amount' => $discountAmount,
                    'discount_qty' => $discountQty,
                    'discount_step' => 0,
                    'apply_to_shipping' => $applyToShipping,
                    'simple_free_shipping' => $simpleFreeShipping,
                    'stop_rules_processing' => 0,
                    'store_labels' => array()
                );

                $id = $this->getRequest()->getParam('rule_id');
                if ($id) {
                    $model->load($id);
                    if ($id != $model->getId()) {
                        throw new \Exception(__('Wrong rule specified.'));
                    }
                }

                if (isset($data['simple_action']) && $data['simple_action'] == 'by_percent'
                    && isset($data['discount_amount'])
                ) {
                    $data['discount_amount'] = min(100, $data['discount_amount']);
                }
                if (isset($data['rule']['conditions'])) {
                    $data['conditions'] = $data['rule']['conditions'];
                }
                if (isset($data['rule']['actions'])) {
                    $data['actions'] = $data['rule']['actions'];
                }
                unset($data['rule']);
                $model->loadPost($data);
                $useAutoGeneration = 0;
                $model->setUseAutoGeneration($useAutoGeneration);

                $model->save();

                $coupon = $model->getPrimaryCoupon();
                $coupon->unsetData('expiration_date');
                $coupon->afterLoad();
                $coupon->setHasDataChanges(true);
                $coupon->setExpirationDate($to_date);
                $coupon->save();

                $result = array(
                    'id' => $model->getRuleId()
                );
                $result = array_merge($result, $dataRecived);
                $result = array('price_rule' => $result);
            }

        } else {
            $result = array(
                'error' => 'Invalide Hmac',
            );
        }
        return $result;
    }
}
