<?php
/**
 * Copyright © Magento, Inc. All rights reserved.
 * See COPYING.txt for license details.
 */

namespace TrustSpot\Lootly\CustomerData;

use Magento\Customer\Helper\Session\CurrentCustomer;
use Magento\Customer\Helper\View;

/**
 * Customer section
 */
class LootlyWidget implements \Magento\Customer\CustomerData\SectionSourceInterface
{
    /**
     * @var CurrentCustomer
     */
    protected $currentCustomer;

    /**
     * @var View
     */
    private $customerViewHelper;
    protected $_scopeConfig;
    protected $storeManager;

    /**
     * @param CurrentCustomer $currentCustomer
     * @param View $customerViewHelper
     */
    public function __construct(
        CurrentCustomer $currentCustomer,
        View $customerViewHelper,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
        \Magento\Store\Model\StoreManagerInterface $storeManager
    )
    {
        $this->currentCustomer = $currentCustomer;
        $this->customerViewHelper = $customerViewHelper;
        $this->_scopeConfig = $scopeConfig;
        $this->storeManager = $storeManager;
    }

    /**
     * {@inheritdoc}
     */
    public function getSectionData()
    {
        if (!$this->currentCustomer->getCustomerId()) {
            return ['customerId' => '', 'customerSignature' => ''];
        }
        $storeId = $this->storeManager->getStore()->getId();
        $scope = 'default';
        if ($storeId) {
            $scope = 'store';
        }
        $customer = $this->currentCustomer->getCustomer();
        $customerId = $customer->getId();
        $secret = $this->_scopeConfig->getValue('lootlyordernotifier/general/api_secret', $scope, $storeId);
        $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
        $MD5Source = $objectManager->get('\TrustSpot\Lootly\Helper\MD5Source');
        $customerSignature = $MD5Source->md($customerId . $secret);

        return [
            'customerId' => $customerId,
            'customerSignature' => $customerSignature,
        ];
    }
}
