<?php

/**
 * TrustSpot_Lootly extension
 *
 * @category       Lootly Extensions
 * @package        TrustSpot_Lootly
 * @copyright      Copyright (c) 2019
 * @license        http://opensource.org/licenses/osl-3.0.php Open Software License (OSL 3.0)
 */

namespace TrustSpot\Lootly\Helper;
/**
 * TrustSpot_Lootly helper
 *
 * @category    Lootly Extensions
 * @package     TrustSpot_Lootly
 * @author      Lootly Inc
 */
class Api extends \TrustSpot\Lootly\Helper\Data
{
    CONST LIVEURL = 'https://lootly.io';
    CONST TESTURL = 'https://lootly.grebola.com';
    /**
     * Verify Settings API URL
     */
    CONST VERIFY_URL = '/integrations/webhooks/magento/key-verify';

    /**
     * Add Review API URL
     */
    //CONST REVIEW_URL = 'https://Lootly.io/api/merchant/add_review';
    CONST SHIPPED_URL = '/integrations/webhooks/magento/orders-fulfilled';
    CONST PROCESSING_URL = '/integrations/webhooks/magento/orders-processing';
    CONST COMPLETED_URL = '/integrations/webhooks/magento/orders-completed';


    CONST APPINSTALL_URL = '/integrations/webhooks/magento/app-installed';
    CONST ORDERPAID_URL = '/integrations/webhooks/magento/orders-paid';
    CONST NEWCUSTOMER_URL = '/integrations/webhooks/magento/customers-create';
    CONST REFUND_URL = '/integrations/webhooks/magento/refunds-create';
    CONST CANCELED_URL = '/integrations/webhooks/magento/orders-cancelled';

    CONST FRONTSCRIPT_URL = '/js/integrations/shopify/script.js?shop=';


    /**
     * Get the verify url of Lootly API
     *
     * @access public
     * @return string
     * @author Lootly Inc
     */
    public function getLiveTestUrl()
    {
        if ($this->scopeConfig->getValue('lootlyordernotifier/general/testmode')) {
            return self::TESTURL;
        } else {
            return self::LIVEURL;
        }
    }

    public function getVerifyURL()
    {
        return self::getLiveTestUrl() . self::VERIFY_URL;
    }

    public function getAppInstallURL()
    {
        return self::getLiveTestUrl() . self::APPINSTALL_URL;
    }

    public function getOrderPaidURL()
    {
        return self::getLiveTestUrl() . self::ORDERPAID_URL;
    }

    public function getOrderShippedURL()
    {
        return self::getLiveTestUrl() . self::SHIPPED_URL;
    }

    public function getOrderProcessingURL()
    {
        return self::getLiveTestUrl() . self::PROCESSING_URL;
    }

    public function getOrderCompletedURL()
    {
        return self::getLiveTestUrl() . self::COMPLETED_URL;
    }

    public function getNewCustomerURL()
    {
        return self::getLiveTestUrl() . self::NEWCUSTOMER_URL;
    }

    public function getRefundURL()
    {
        return self::getLiveTestUrl() . self::REFUND_URL;
    }

    public function getCanceledURL()
    {
        return self::getLiveTestUrl() . self::CANCELED_URL;
    }


    public function getScriptUrl()
    {
        return self::getLiveTestUrl() . self::FRONTSCRIPT_URL . $this->_urlBuilder->getBaseUrl();
    }

    /**
     * Verify Lootly API Connection
     *
     * @access public
     * @param array
     * @return bool
     * @author Lootly Inc
     */
    public function verify($dataVerify = array())
    {
        $storeId = $this->_getRequest()->getParam('store');
        $verifyUrl = $this->getVerifyURL();
        $key = $this->getApiKey($storeId);
        if (isset($dataVerify['key'])) {
            $key = $dataVerify['key'];
        }
        if (!$key) {
            return false;
        }
        $secret = $this->getApiSecret($storeId);
        if (isset($dataVerify['secret'])) {
            $secret = $dataVerify['secret'];
        }
        $data = array(
            'key' => $key,
            't' => time()
        );
        ksort($data);
        $data['hmac'] = base64_encode(hash_hmac('sha256', json_encode($data), $secret, true));

        $result = $this->call($data, $verifyUrl);
        $status = $result['status'];
        return ($status > 0) ? $status : false;
    }


    public function installApp($apiKey, $apiSecret)
    {
        $storeIds = [];
        $stores = $this->storeManager->getStores();
        foreach ($stores as $store) {
            $key = $this->getApiKey($store->getId());
            if ($apiKey==$key){
                $installedUrl = $this->storeManager->getStore($store->getId())->getBaseUrl();
                break;
            }
        }
        $installedUrl = str_replace('/index.php/', '/', $installedUrl);
        $requestParams = array(
            "shop_url" => $installedUrl,
            "api_endpoint" => $this->_getUrl('lootly/create_coupon/index', ['_nosid' => true]),
            "key" => $apiKey,
        );
        ksort($requestParams);
        $requestParams["hmac"] = base64_encode(hash_hmac('sha256', json_encode($requestParams), $apiSecret, true));
        $this->call($requestParams, $this->getAppInstallURL());
    }

    /**
     * Call Lootly API
     *
     * @access public
     * @return object
     * @author Lootly Inc
     */

    public function call($data, $url)
    {
        $dataString = json_encode($data);
        $curl = curl_init($url);
        curl_setopt($curl, CURLOPT_CUSTOMREQUEST, "POST");
        curl_setopt($curl, CURLOPT_HTTPHEADER, array('Content-Type: application/json', 'Content-Length: ' . strlen($dataString)));
        curl_setopt($curl, CURLOPT_POSTFIELDS, $dataString); // Insert the data
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_TIMEOUT, 10);
        $result = curl_exec($curl);
        $responseInfo = curl_getinfo($curl);
        curl_close($curl);
        $this->_logger->debug($result);
        if (in_array($responseInfo['http_code'], array('200', '201'))) {
            $res = array('status' => 1);
        } else {
            $res = array('status' => 0);
        }

        return $res;
    }

    /**
     * @param \Magento\Sales\Model\Order $order
     * @param null $targetStatus
     */
    public function sendToApi($order, $targetStatus = null)
    {
        $storeId = $order->getStoreId();
        $model = $this->orderNotifierEntryFactory->create();
        $entries = $model->getCollection()->addFieldToFilter('order_id', $order->getIncrementId());

        if ($entries->getSize() <= 0) {
            $orderId = $order->getIncrementId();
            $secretKey = $this->getApiSecret($storeId);

            /* Get Order Total, Shipping country, zipcode, Order Coupon code */
            $orderTotal = $order->getBaseGrandTotal();
            $_shippingAddress = $order->getShippingAddress();
            $shippingZipcode = $_shippingAddress->getPostcode();
            $shippingCountry = $_shippingAddress->getCountryId();
            $taxAmount = $order->getTaxAmount();
            $couponCode = $order->getCouponCode();
            $discount_codes = array();
            if ($couponCode) {
                $discount_codes[] = array('code' => $couponCode);
            }
            $customerId = $order->getCustomerId();
            if (!$customerId) {
                $websiteId = $order->getStore()->getWebsite()->getId();
                try {
                    $customer = $this->customerRepository->get($order->getCustomerEmail(), $websiteId);
                    $customerId = $customer->getId();
                } catch (\Exception $e) {

                }
            }
            $customerInfo = array(
                'email' => $order->getCustomerEmail()
            );
            $firstname = $order->getCustomerFirstname();
            if ($firstname) {
                $customerInfo['first_name'] = $firstname;
            } else {
                $customerInfo['first_name'] = $order->getBillingAddress()->getFirstname();
            }
            $last_name = $order->getCustomerLastname();
            if ($last_name) {
                $customerInfo['last_name'] = $last_name;
            } else {
                $customerInfo['last_name'] = $order->getBillingAddress()->getLastname();
            }
            if ($customerId) {
                $customerInfo['id'] = $customerId;
                $address = $order->getShippingAddress();
                if ($address && $address->getId()) {
                    $customerInfo['default_address'] = array(
                        'zip' => $address->getPostcode(),
                        'country' => $address->getCountryId()
                    );
                }
            }
            $customerDob = $order->getCustomerDob();
            if ($customerDob && $customerDob != '0000-00-00 00:00:00' && $customerDob != '0000-00-00') {
                $customerInfo['birthday'] = $customerDob;
            }
            $discount = abs($order->getDiscountAmount());
            $subtotal = $order->getSubtotal() - $discount;
            $productsData = [];
            foreach ($order->getAllVisibleItems() as $item){
                /** @var \Magento\Catalog\Model\Product $product */
                $product = $item->getProduct();
                $categories = [];
                $cats = $product->getCategoryCollection();
                $cats->addAttributeToSelect('name');
                foreach ($cats as $category){
                    $categories[] = [
                        'category_name' => $category->getName(),
                        'category_id' => $category->getId()
                    ];
                }
                $productData = [];
                $productData['product_id'] = $item->getProductId();
                $productData['product_name'] = $item->getName();
                $productData['product_price'] = $item->getBasePrice();
                $productData['quantity'] = $item->getQtyOrdered();
                $productData['categories'] = $categories;
                $productsData[] = $productData;
            }
            $apiData = array(
                "id" => $order->getIncrementId(),
                'total_price' => number_format($orderTotal, 2, '.', ''),
                'total_tax' => number_format($taxAmount, 2, '.', ''),
                'total_discounts' => number_format($discount, 2, '.', ''),
                'subtotal_price' => number_format($subtotal, 2, '.', ''),
                'taxes_included' => '0',
                'discount_codes' => $discount_codes,
                'customer' => $customerInfo,
                "key" => $this->getApiKey($storeId),
                "products" => $productsData,
                'ip_address' => $order->getRemoteIp()
            );
            /* Pass Coupon code to the API call If coupon code was used */
            if (!empty($couponCode)) {
                $apiData['coupon_code'] = $couponCode;
            }

            ksort($apiData);
            $apiData["hmac"] = base64_encode(hash_hmac('sha256', json_encode($apiData), $secretKey, true));

            $url = $this->getOrderProcessingURL();
            if ($targetStatus == 'shipped') {
                $url = $this->getOrderShippedURL();
            } elseif ($targetStatus == 'invoiced') {
                $url = $this->getOrderPaidURL();
            } elseif ($targetStatus == 'complete') {
                $url = $this->getOrderCompletedURL();
            }
            $result = $this->call($apiData, $url);

            if ($result['status'] == '1') {
                /*Mage::getSingleton('adminhtml/session')->addSuccess('Order info has been sent to Lootly.');*/
                $data = array(
                    'order_id' => $order->getIncrementId()
                );
                $model->setData($data);
                $model->save();
            } else {
                /*Mage::getSingleton('adminhtml/session')->addError('Error! Order info was not sent to Lootly.');*/
            }
        }
    }

    public function processCanceled($order)
    {
        /** @var \Magento\Sales\Model\Order $order */
        $storeId = $order->getStoreId();
        $apiData = array(
            'id' => $order->getIncrementId(),
            "key" => $this->getApiKey($storeId),
        );
        $secretKey = $this->getApiSecret($storeId);

        ksort($apiData);
        $apiData["hmac"] = base64_encode(hash_hmac('sha256', json_encode($apiData), $secretKey, true));

        $url = $this->getCanceledURL();
        $result = $this->call($apiData, $url);
    }
}
