<?php

/**
 * TrustSpot_Lootly extension
 *
 * @category       Lootly Extensions
 * @package        TrustSpot_Lootly
 * @copyright      Copyright (c) 2019
 * @license        http://opensource.org/licenses/osl-3.0.php Open Software License (OSL 3.0)
 */

namespace TrustSpot\Lootly\Helper;

use Magento\Framework\App\Helper\Context;
use Magento\Framework\Module\ModuleListInterface;

/**
 * TrustSpot_Lootly helper
 *
 * @category    Lootly Extensions
 * @package     TrustSpot_Lootly
 * @author      Lootly Inc
 */
class Data extends \Magento\Framework\App\Helper\AbstractHelper
{

    /**
     * Lootly Signup page URL
     */
    CONST SIGNUP_URL = 'https://lootly.io/signup';

    /**
     * Lootly Login page URL
     */
    CONST LOGIN_URL = 'https://lootly.io/login';

    protected $_moduleList;
    protected $orderNotifierEntryFactory;
    protected $storeManager;
    protected $customerRepository;

    public function __construct(
        Context $context,
        ModuleListInterface $moduleList,
        \TrustSpot\Lootly\Model\EntryFactory $orderNotifierEntryFactory,
        \Magento\Store\Model\StoreManagerInterface $storeManager,
        \Magento\Customer\Api\CustomerRepositoryInterface $customerRepository
    )
    {
        $this->customerRepository = $customerRepository;
        $this->orderNotifierEntryFactory = $orderNotifierEntryFactory;
        $this->_moduleList = $moduleList;
        $this->storeManager = $storeManager;
        parent::__construct($context);
    }

    public function getConfigValue($code, $storeId)
    {
        $scope = 'default';
        if ($storeId > 0) {
            $scope = 'store';
        }
        return $this->scopeConfig->getValue($code, $scope, $storeId);
    }

    /**
     * Retrieve Email Address
     *
     * @access public
     * @return string
     * @author Lootly Inc
     */
    public function getEmail($storeId = null)
    {
        return $this->getConfigValue('lootlyordernotifier/general/email', $storeId);
    }

    /**
     * Get API Key
     *
     * @access public
     * @return string
     * @author Lootly Inc
     */
    public function getApiKey($storeId = null)
    {
        return $this->getConfigValue('lootlyordernotifier/general/api_key', $storeId);
    }

    /**
     * Get API Secret
     *
     * @access public
     * @return string
     * @author Lootly Inc
     */
    public function getApiSecret($storeId = null)
    {
        return $this->getConfigValue('lootlyordernotifier/general/api_secret', $storeId);
    }

    /**
     * Get Order Status
     *
     * @access public
     * @return string
     * @author Lootly Inc
     */
    public function getTargetOrderStatus($storeId = null)
    {
        return $this->getConfigValue('lootlyordernotifier/general/order_status', $storeId);
    }

    /**
     * Get Lootly Signup page URL
     *
     * @access public
     * @return string
     * @author Lootly Inc
     */
    public function getSignupURL()
    {
        return self::SIGNUP_URL;
    }

    public function getExtensionVersion()
    {
        $module = $this->_moduleList->getOne('TrustSpot_Lootly');
        return (string)$module['setup_version'];
    }

    public function getRefundOrderStatus($storeId = null)
    {
        return $this->getConfigValue('lootlyordernotifier/general/refund_status', $storeId);
    }
}
