<?php

/**
 * TrustSpot_Lootly extension
 * @category       Lootly Extensions
 * @package        TrustSpot_Lootly
 * @copyright      Copyright (c) 2015
 * @license        http://opensource.org/licenses/osl-3.0.php Open Software License (OSL 3.0)
 */

namespace TrustSpot\Lootly\Observer;

class Refund implements \Magento\Framework\Event\ObserverInterface
{
    /**
     * @var \TrustSpot\Lootly\Helper\Data
     */
    protected $orderNotifierHelper;

    /**
     * @var \TrustSpot\Lootly\Helper\Api
     */
    protected $orderNotifierApiHelper;

    public function __construct(
        \TrustSpot\Lootly\Helper\Data $orderNotifierHelper,
        \TrustSpot\Lootly\Helper\Api $orderNotifierApiHelper
    )
    {
        $this->orderNotifierHelper = $orderNotifierHelper;
        $this->orderNotifierApiHelper = $orderNotifierApiHelper;
    }

    public function execute(
        \Magento\Framework\Event\Observer $observer
    )
    {
        /** @var \Magento\Sales\Model\Order\Creditmemo $creditmemo */
        $creditmemo = $observer->getData('creditmemo');
        /** @var \Magento\Sales\Model\Order $order */
        $order = $creditmemo->getOrder();
        $refundTotal = $creditmemo->getGrandTotal();
        $apiData = array(
            'id' => $order->getIncrementId(),
            'transactions' => array(
                array(
                    'amount' => $refundTotal
                )
            ),
            "key" => $this->orderNotifierApiHelper->getApiKey(),
        );
        $secretKey = $this->orderNotifierApiHelper->getApiSecret();

        ksort($apiData);
        $apiData["hmac"] = base64_encode(hash_hmac('sha256', json_encode($apiData), $secretKey, true));

        $url = $this->orderNotifierApiHelper->getRefundURL();
        $result = $this->orderNotifierApiHelper->call($apiData, $url);
    }
}
