<?php

/**
 * TrustSpot_Lootly extension
 * @category       Lootly Extensions
 * @package        TrustSpot_Lootly
 * @copyright      Copyright (c) 2015
 * @license        http://opensource.org/licenses/osl-3.0.php Open Software License (OSL 3.0)
 */

namespace TrustSpot\Lootly\Observer;

class SalesOrderSaveAfter implements \Magento\Framework\Event\ObserverInterface
{
    /**
     * @var \TrustSpot\Lootly\Helper\Data
     */
    protected $orderNotifierHelper;

    /**
     * @var \TrustSpot\Lootly\Helper\Api
     */
    protected $orderNotifierApiHelper;

    public function __construct(
        \TrustSpot\Lootly\Helper\Data $orderNotifierHelper,
        \TrustSpot\Lootly\Helper\Api $orderNotifierApiHelper
    )
    {
        $this->orderNotifierHelper = $orderNotifierHelper;
        $this->orderNotifierApiHelper = $orderNotifierApiHelper;
    }

    public function execute(
        \Magento\Framework\Event\Observer $observer
    )
    {
        $order = $observer->getOrder();
        $_helper = $this->orderNotifierHelper;
        $storeId = $order->getStoreId();
        $targetStatus = $_helper->getTargetOrderStatus($storeId);
        $refundStatus = $_helper->getRefundOrderStatus($storeId);

        if ($targetStatus != '' && $targetStatus == $order->getState()) {
            $this->orderNotifierApiHelper->sendToApi($order, $order->getState());
        }

        if ($refundStatus != '' && $refundStatus == $order->getStatus()) {
            $this->orderNotifierApiHelper->processCanceled($order);
        }

        return $this;
    }
}
